module Cats
  module Core
    class AccessController < ApplicationController
      skip_before_action :authenticate, only: [:login]

      def login
        user = Cats::Core::User.find_by(email: auth_params[:email])
        if user
          if user.authenticate(auth_params[:password])
            roles = user.roles.map(&:name)

            unless roles.count.positive?
              render json: { error: 'User has no roles.' }, status: :unprocessable_entity
              return
            end

            payload = {
              id: user.id, email: user.email, first_name: user.first_name, last_name: user.last_name, roles: roles
            }
            jwt = Cats::Core::TokenAuthService.issue(payload)
            render json: { token: jwt, user: payload }
          else
            rener json: { error: 'Invalid username or password.' }, status: 400
          end
        else
          render json: { error: 'User does not exist.' }, status: 400
        end
      end

      private

      def auth_params
        params.require(:auth).permit(:email, :password)
      end
    end
  end
end
