module Cats
  module Core
    class CommodityCategoriesController < ApplicationController
      before_action :set_commodity_category, only: %i[show update]

      def index
        commodity_categories = Cats::Core::CommodityCategory.all.reject(&:has_children?)
        data = ActiveModelSerializers::SerializableResource.new(commodity_categories)
        render json: { success: true, data: data }
      end

      def show
        data = ActiveModelSerializers::SerializableResource.new(@commodity_category)
        render json: { success: true, data: data }
      end

      def children
        parent = Cats::Core::CommodityCategory.find(params[:id])
        data = ActiveModelSerializers::SerializableResource.new(parent.children)
        render json: { success: true, data: data }
      end

      def create
        obj = Cats::Core::CommodityCategory.new(model_params)
        if obj.save
          data = ActiveModelSerializers::SerializableResource.new(obj)
          render json: { success: true, data: data }, status: :created
        else
          render json: { success: false, error: obj.errors.full_messages[0] }, status: :unprocessable_entity
        end
      end

      def update
        if @commodity_category.update(model_params)
          data = ActiveModelSerializers::SerializableResource.new(@commodity_category)
          render json: { success: true, data: data }
        else
          render json: {
            success: false, error: @commodity_category.errors.full_messages[0]
          }, status: :unprocessable_entity
        end
      end

      private

      def set_commodity_category
        @commodity_category = Cats::Core::CommodityCategory.find(params[:id])
      end

      def model_params
        params.require('payload').permit(:code, :name, :description, :parent_id)
      end
    end
  end
end
