module Cats
  module Core
    class LocationsController < ApplicationController
      before_action :set_location, only: %i[show update]

      def index
        locations = Cats::Core::Location.where(location_type: params[:location_type])
        data = ActiveModelSerializers::SerializableResource.new(locations)
        render json: { success: true, data: data }
      end

      def show
        data = ActiveModelSerializers::SerializableResource.new(@location)
        render json: { success: true, data: data }
      end

      def children
        parent = Cats::Core::Location.find(params[:id])
        data = ActiveModelSerializers::SerializableResource.new(parent.children)
        render json: { success: true, data: data }
      end

      def create
        obj = Cats::Core::Location.new(model_params)
        if obj.save
          data = ActiveModelSerializers::SerializableResource.new(obj)
          render json: { success: true, data: data }, status: :created
        else
          render json: { success: false, error: obj.errors.full_messages[0] }, status: :unprocessable_entity
        end
      end

      def update
        if @location.update(model_params)
          data = ActiveModelSerializers::SerializableResource.new(@location)
          render json: { success: true, data: data }
        else
          render json: { success: false, error: @location.errors.full_messages[0] }, status: :unprocessable_entity
        end
      end

      private

      def set_location
        @location = Cats::Core::Location.find(params[:id])
      end

      def model_params
        params.require(:payload).permit(:name, :location_type, :description, :parent_id)
      end
    end
  end
end
