module Cats
  module Core
    class UnitOfMeasuresController < ApplicationController
      before_action :set_unit_of_measure, only: %i[show update]

      def index
        data = ActiveModelSerializers::SerializableResource.new(Cats::Core::UnitOfMeasure.all)
        render json: { success: true, data: data }
      end

      def show
        data = ActiveModelSerializers::SerializableResource.new(@unit_of_measure)
        render json: { success: true, data: data }
      end

      def create
        obj = Cats::Core::UnitOfMeasure.new(model_params)
        if obj.save
          data = ActiveModelSerializers::SerializableResource.new(obj)
          render json: { success: true, data: data }, status: :created
        else
          render json: { success: false, error: obj.errors.full_messages[0] }, status: :unprocessable_entity
        end
      end

      def update
        if @unit_of_measure.update(model_params)
          data = ActiveModelSerializers::SerializableResource.new(@unit_of_measure)
          render json: { success: true, data: data }
        else
          render json: {
            success: false,
            error: @unit_of_measure.errors.full_messages[0]
          }, status: :unprocessable_entity
        end
      end

      private

      def set_unit_of_measure
        @unit_of_measure = Cats::Core::UnitOfMeasure.find(params[:id])
      end

      def model_params
        params.require(:payload).permit(:abbreviation, :name, :unit_type)
      end
    end
  end
end
