module Cats
  module Core
    class AllocationItem < ApplicationRecord
      belongs_to :allocation
      belongs_to :destination, class_name: 'Cats::Core::Location'

      validates :from, :to, presence: true
      validates :quantity, presence: true, numericality: { greater_than: 0 }
      validate :validate_source_and_destination

      def validate_source_and_destination
        return unless allocation.present? && destination.present?

        errors.add(:base, 'source and destination cannot be the same') if allocation.source == destination
      end

      delegate(:name, to: :destination, prefix: true)
      delegate(:reference_no, to: :allocation, prefix: true)
      delegate(:location_type, to: :destination, prefix: true)
    end
  end
end
