module Cats
  module Core
    class Commodity < ApplicationRecord
      after_initialize :set_approved

      # Commodity statuses
      GOOD = 'Good'.freeze
      DAMAGED = 'Damaged'.freeze
      COMMODITY_STATUSES = [GOOD, DAMAGED].freeze

      # Arrival Statuses
      AT_SOURCE = 'At Source'.freeze
      IN_TRANSIT = 'In Transit'.freeze
      ARRIVED = 'Arrived'.freeze
      ARRIVAL_STATUSES = [AT_SOURCE, IN_TRANSIT, ARRIVED].freeze

      belongs_to :unit_of_measure
      belongs_to :source, polymorphic: true
      belongs_to :commodity_category

      validates :best_use_before, presence: true
      validates :batch_no, presence: true, uniqueness: true
      validates :quantity, presence: true, numericality: { greater_than: 0 }
      validates :volume_per_metric_ton, numericality: { greater_than: 0, allow_nil: true }
      validates :arrival_status, presence: true, inclusion: { in: ARRIVAL_STATUSES }

      delegate(:abbreviation, to: :unit_of_measure, prefix: true)
      delegate(:name, to: :commodity_category, prefix: true)
      delegate(:reference_no, to: :source, prefix: true)

      def set_approved
        return unless new_record?

        self.approved = false
      end
    end
  end
end
