module Cats
  module Core
    class Donation < ApplicationRecord
      CASH = 'Cash'.freeze
      KIND = 'Kind'.freeze
      DONATION_TYPES = [CASH, KIND].freeze

      belongs_to :donor
      belongs_to :hrp
      belongs_to :currency, optional: true

      validates :reference_no, :donation_type, :donated_on, presence: true
      validates :reference_no, uniqueness: true
      validates :donation_type, inclusion: { in: DONATION_TYPES }
      validates :currency, :amount, presence: true, if: -> { donation_type == CASH }

      delegate(:name, to: :donor, prefix: true)
      delegate(:reference_no, to: :hrp, prefix: true)
    end
  end
end
