module Cats
  module Core
    class Location < ApplicationRecord
      REGION = 'Region'.freeze
      ZONE = 'Zone'.freeze
      WOREDA = 'Woreda'.freeze
      KEBELE = 'Kebele'.freeze
      HUB = 'Hub'.freeze
      WAREHOUSE = 'Warehouse'.freeze

      LOCATION_TYPES = [REGION, ZONE, WOREDA, KEBELE, HUB, WAREHOUSE].freeze

      has_ancestry

      validates :name, :location_type, presence: true
      validates :location_type, inclusion: { in: LOCATION_TYPES }
      validate :validate_location_parent

      def validate_location_parent
        parents = {
          REGION => [],
          ZONE => [REGION],
          WOREDA => [REGION, ZONE],
          KEBELE => [REGION, ZONE, WOREDA],
          HUB => [REGION, ZONE, WOREDA, KEBELE],
          WAREHOUSE => [REGION, ZONE, WOREDA, KEBELE, HUB]
        }

        return if location_type.nil? || location_type.empty?

        return if location_type == REGION && parent.nil?

        # Check that parent is not nil for locations other than region
        errors.add(:location, 'parent cannot be empty') if location_type != REGION && parent.nil?

        # At this point, we may return if parent.nil? is true
        # because the above statement already took care of that.
        return unless parent

        return if parents[location_type].include?(parent.location_type)

        # If we reach this far then it means that we have assigned a
        # wrong type of parent to our location.
        errors.add(:location, "cannot have #{parent.location_type} as parent")
      end
    end
  end
end
