module Cats
  module Core
    class Receipt < ApplicationRecord
      # Receipt status
      DRAFT = 'Draft'.freeze
      ACCEPTED = 'Accepted'.freeze
      DECLINED = 'Declined'.freeze
      RECEIPT_STATUSES = [DRAFT, ACCEPTED, DECLINED].freeze

      belongs_to :dispatch
      belongs_to :prepared_by, class_name: 'Cats::Core::User'

      validates :quantity, :commodity_status, :status, presence: true
      validates :quantity, numericality: { greater_than: 0 }
      validates :status, inclusion: { in: RECEIPT_STATUSES }
      validates :commodity_status, inclusion: { in: Cats::Core::Commodity::COMMODITY_STATUSES }
    end
  end
end
