module Cats
  module Core
    class Store < ApplicationRecord
      belongs_to :warehouse, class_name: 'Cats::Core::Location'
      belongs_to :store_keeper, class_name: 'Cats::Core::User'
      has_many :stacks

      validates :name, :length, :width, :height, presence: true
      validates :length, :width, :height, numericality: { greater_than: 0 }
      validates :gangway_length, :gangway_width, :gangway_corner_dist, presence: true, if: :has_gangway?
      validates :gangway_length,
                :gangway_width,
                :gangway_corner_dist,
                numericality: { greater_than: 0 },
                allow_nil: true
      validate :validate_location

      after_save :update_store_keeper

      def validate_location
        return if warehouse.nil?

        errors.add(:warehouse, 'must be a valid warehouse') unless warehouse.location_type == Location::WAREHOUSE
      end

      def store_keeper_name
        "#{store_keeper.first_name} #{store_keeper.last_name}"
      end

      def update_store_keeper
        return unless store_keeper_id_previously_changed?

        unless id_previously_changed?
          old_storekeeper = Cats::Core::User.find(store_keeper_id_previously_was)
          old_storekeeper.details['stores'].delete(id)
          old_storekeeper.save
        end
        if store_keeper.details.key?('stores')
          store_keeper.details['stores'] << id
        else
          store_keeper.details['stores'] = [id]
        end
        store_keeper.save
      end

      delegate(:phone_number, to: :store_keeper, prefix: true)
    end
  end
end
