module Cats
  module Core
    class Transaction < ApplicationRecord
      self.abstract_class = true

      # Transaction statuses
      DRAFT = 'Draft'.freeze
      COMMITTED = 'Committed'.freeze
      STATUSES = [DRAFT, COMMITTED].freeze

      validates :transaction_date, :quantity, :status, presence: true
      validates :quantity, numericality: { greater_than: 0 }
      validates :status, inclusion: { in: STATUSES }
      validate :validate_quantity

      def validate_quantity
        return unless quantity.present? && source.present?

        errors.add(:quantity, 'cannot be more than source quantity') if quantity > source.quantity
      end

      def commit
        Transaction.transaction do
          source.quantity -= quantity
          destination.quantity += quantity
          source.save
          destination.save
        end
      end
    end
  end
end
