module Cats
  module Core
    class User < ApplicationRecord
      has_secure_password
      rolify role_cname: 'Cats::Core::Role'

      belongs_to :application_module
      has_many :notifications, as: :recipient

      validates :password, length: { minimum: 6 }, if: proc { |u| u.password.present? }, on: :update
      validates :first_name, :last_name, :email, presence: true
      validates :password, presence: true, length: { minimum: 6 }, on: :create
      validates :email, uniqueness: true, format: { with: URI::MailTo::EMAIL_REGEXP }
      validate :validate_phone_number

      def warehouses
        return unless details.key?('warehouses')

        Cats::Core::Location.where(id: details['warehouses'], location_type: Cats::Core::Location::WAREHOUSE)
      end

      def stores
        return unless details.key?('stores')

        Cats::Core::Store.where(id: details['stores'])
      end

      def validate_phone_number
        return unless phone_number.present?

        return if phone_number.length == 10 && /0\d{9}/.match(phone_number)

        return if phone_number.length == 12 && /251\d{9}/.match(phone_number)

        errors.add(:phone_number, 'must be 10 or 12 digits and should match with local phone pattern')
      end
    end
  end
end
