module Cats
  module Core
    class NotificationService
      NOTIFICATION_RULES = {
        dispatch_plan: {
          application: 'CATS-WH',
          notification: 'DispatchPlanNotification',
          params: [:way_bill],
          recipients: [:warehouse_manager]
        },
        receipt_plan: {
          application: 'CATS-WH',
          notification: 'ReceiptPlanNotification',
          params: [:receipt_plan],
          recipients: [:store_keeper]
        }
      }.freeze

      def initialize(rules, code, params)
        @rules = rules
        @code = code
        @params = params
      end

      def create_notifier(rule)
        clazz = rule[:notification].constantize
        clazz.with(**@params)
      end

      def notify
        rule = @rules[@code]
        notifier = create_notifier(rule)
        app_code = rule[:application]
        roles = rule[:recipients]

        users = Cats::Core::User.joins(:application_module)
                                .where(application_module: { prefix: app_code }).with_all_roles(*roles)
        notifier.deliver(users)
      end
    end
  end
end
